﻿/*
      Copyright (C) 2009 Bepu Entertainment LLC

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics;

namespace BEPUphysicsDemos
{
    /// <summary>
    /// Defines a pointmass style entity.
    /// </summary>
    public class VertexEntity : Entity
    {
        /// <summary>
        /// Dynamic constructor.
        /// </summary>
        /// <param name="position">Position of the vertex.</param>
        /// <param name="mass">Mass of the vertex.</param>
        public VertexEntity(Vector3 position, float mass)
        {
            moveTo(position);
            this.mass = mass;
            initialize(true);
        }

        /// <summary>
        /// Kinematic constructor.
        /// </summary>
        /// <param name="position">Position of the vertex.</param>
        public VertexEntity(Vector3 position)
        {
            moveTo(position);
            initialize(false);
        }

        /// <summary>
        /// Initializes the entity.  Called when the entity is constructed and later if becomeKinematic/Dynamic is called.
        /// </summary>
        /// <param name="physicallySimulated">Whether or not the entity is physically simulated.</param>
        protected override void initialize(bool physicallySimulated)
        {
            if (physicallySimulated)
            {
                findBoundingBox(0);
                volume = 0;
                density = 0;

                localSpaceInertiaTensorInverse = Toolbox.zeroMatrix;

                initializePhysicalData();
            }
            else
            {
                findBoundingBox(0);
                volume = 0;
                initializeNonDynamicData();
            }
            myMaximumRadius = 0;
            myMaximumRadiusFromCenterPosition = 0;

        }

        /**
         * 
         * The following methods are used by the collision detection system.
         * Since particles are just tiny points, the extreme point is the position.
         * 
         * The 'margin' calculations take into account the entity's collisionMargin. All entities have these, though you can set them on a per-entity basis.
         * If you set the collision margin for the vertices to 0, the extra margin code becomes pointless.
         * 
         * Also, be careful of what is included in these methods.  They are are some of the most frequently called functions in the entire engine.
         * 
         */

        public override void getExtremePoint(ref Vector3 d, ref Vector3 positionToUse, ref Quaternion orientationToUse, float margin, out Vector3 extremePoint)
        {
            extremePoint = positionToUse;

            if (margin > 0)
            {
                Vector3 marginExtra;
                Vector3.Normalize(ref d, out marginExtra);
                Vector3.Multiply(ref marginExtra, margin, out marginExtra);
                Vector3.Add(ref extremePoint, ref marginExtra, out extremePoint);
            }
        }

        public override void getExtremePoints(ref Vector3 d, out Vector3 min, out Vector3 max, float margin)
        {
            min = internalCenterPosition;
            max = min;

            if (margin > 0)
            {
                Vector3 marginExtra;
                Vector3.Normalize(ref d, out marginExtra);
                Vector3.Multiply(ref marginExtra, margin, out marginExtra);
                Vector3.Add(ref max, ref marginExtra, out max);

                Vector3.Subtract(ref min, ref marginExtra, out min);
            }
        }

        public override void getExtremePoints(ref Vector3 d, ref Vector3 positionToUse, ref Quaternion orientationToUse, out Vector3 min, out Vector3 max, float margin)
        {
            min = positionToUse;
            max = min;

            if (margin > 0)
            {
                Vector3 marginExtra;
                Vector3.Normalize(ref d, out marginExtra);
                Vector3.Multiply(ref marginExtra, margin, out marginExtra);
                Vector3.Add(ref max, ref marginExtra, out max);

                Vector3.Subtract(ref min, ref marginExtra, out min);
            }
        }
    }
}
